; Copyright (c) 2012, Tom Van Cutsem, Vrije Universiteit Brussel
; All rights reserved.
;
; Redistribution and use in source and binary forms, with or without
; modification, are permitted provided that the following conditions are met:
;    * Redistributions of source code must retain the above copyright
;      notice, this list of conditions and the following disclaimer.
;    * Redistributions in binary form must reproduce the above copyright
;      notice, this list of conditions and the following disclaimer in the
;      documentation and/or other materials provided with the distribution.
;    * Neither the name of the Vrije Universiteit Brussel nor the
;      names of its contributors may be used to endorse or promote products
;      derived from this software without specific prior written permission.
;
;THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
;ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
;WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
;DISCLAIMED. IN NO EVENT SHALL VRIJE UNIVERSITEIT BRUSSEL BE LIABLE FOR ANY
;DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
;(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES
;LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
;ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
;(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
;SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

; Vector swap example

(defn make-random-state-vector [size]
  (vec (map ref (range size))))

(def state (make-random-state-vector 10))

(defn read-state [state]
  (dosync (map deref state)))

(defn random-element [state]
  (nth state (rand-int (count state))))

(defn swap-items [state]
  (let [r1 (random-element state)
        r2 (random-element state)]
    (dosync
      (let [temp @r1]
          (ref-set r1 @r2)
          (ref-set r2 temp)))))

(defn swapper-thread [n state]
  (new Thread
       (fn []
         (dotimes [_ n]
           (swap-items state)))))

(defn do-swap []
  (let [T1 (swapper-thread 100 state)
        T2 (swapper-thread 100 state)]
    (.start T1)
    (.start T2)
    (.join T1)
    (.join T2)))
    
; call (read-state state)
; call (do-swap)
; call (read-state state) again
; items will have been permuted by 2 threads, but all elements will still be there